@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -119,7 +119,7 @@ class GetVersion(BaseCommand):
     must_be_initialized = False
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <get_version> command.
+        """Handles <get_version> command.
 
         Return:
             Response string for <get_version> command.
@@ -162,21 +162,21 @@ class GetVersion(BaseCommand):
 
 
 GVMCG_TITLES = [
-    'cpu-*',
+    'cpu-.*',
     'proc',
     'mem',
     'swap',
     'load',
-    'df-*',
+    'df-.*',
     'disk-sd[a-z][0-9]-rw',
     'disk-sd[a-z][0-9]-load',
     'disk-sd[a-z][0-9]-io-load',
-    'interface-eth*-traffic',
-    'interface-eth*-err-rate',
-    'interface-eth*-err',
-    'sensors-*_temperature-*',
-    'sensors-*_fanspeed-*',
-    'sensors-*_voltage-*',
+    'interface-eth.*-traffic',
+    'interface-eth.*-err-rate',
+    'interface-eth.*-err',
+    'sensors-.*_temperature-.*',
+    'sensors-.*_fanspeed-.*',
+    'sensors-.*_voltage-.*',
     'titles',
 ]  # type: List[str]
 
@@ -192,7 +192,7 @@ class GetPerformance(BaseCommand):
     must_be_initialized = False
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <get_performance> command.
+        """Handles <get_performance> command.
 
         @return: Response string for <get_performance> command.
         """
@@ -207,7 +207,7 @@ class GetPerformance(BaseCommand):
             except ValueError:
                 raise OspdCommandError(
                     'Start argument must be integer.', 'get_performance'
-                )
+                ) from None
 
             cmd.append(start)
 
@@ -217,7 +217,7 @@ class GetPerformance(BaseCommand):
             except ValueError:
                 raise OspdCommandError(
                     'End argument must be integer.', 'get_performance'
-                )
+                ) from None
 
             cmd.append(end)
 
@@ -236,7 +236,7 @@ class GetPerformance(BaseCommand):
         except (subprocess.CalledProcessError, OSError) as e:
             raise OspdCommandError(
                 'Bogus get_performance format. %s' % e, 'get_performance'
-            )
+            ) from None
 
         return simple_response_str(
             'get_performance', 200, 'OK', output.decode()
@@ -249,7 +249,7 @@ class GetScannerDetails(BaseCommand):
     must_be_initialized = True
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <get_scanner_details> command.
+        """Handles <get_scanner_details> command.
 
         @return: Response string for <get_scanner_details> command.
         """
@@ -281,7 +281,7 @@ class DeleteScan(BaseCommand):
     must_be_initialized = False
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <delete_scan> command.
+        """Handles <delete_scan> command.
 
         @return: Response string for <delete_scan> command.
         """
@@ -313,7 +313,7 @@ class GetVts(BaseCommand):
     must_be_initialized = True
 
     def handle_xml(self, xml: Element) -> Iterator[bytes]:
-        """ Handles <get_vts> command.
+        """Handles <get_vts> command.
         Writes the vt collection on the stream.
         The <get_vts> element accept two optional arguments.
         vt_id argument receives a single vt id.
@@ -347,7 +347,7 @@ class GetVts(BaseCommand):
                 )
             except OspdCommandError as filter_error:
                 self._daemon.vts.is_cache_available = True
-                raise OspdCommandError(filter_error)
+                raise filter_error
 
         if not version_only:
             vts_selection = self._daemon.get_vts_selection_list(
@@ -389,7 +389,7 @@ class StopScan(BaseCommand):
     must_be_initialized = True
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <stop_scan> command.
+        """Handles <stop_scan> command.
 
         @return: Response string for <stop_scan> command.
         """
@@ -411,17 +411,18 @@ class StopScan(BaseCommand):
 
 class GetScans(BaseCommand):
     name = 'get_scans'
-    description = 'List the scans in buffer.'
+    description = 'Get information about a scan in buffer.'
     attributes = {
-        'scan_id': 'ID of a specific scan to get.',
+        'scan_id': 'Mandatory ID of a specific scan to get.',
         'details': 'Whether to return the full scan report.',
         'pop_results': 'Whether to remove the fetched results.',
         'max_results': 'Maximum number of results to fetch.',
+        'progress': 'Whether to return a detailed scan progress',
     }
     must_be_initialized = False
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <get_scans> command.
+        """Handles <get_scans> command.
 
         @return: Response string for <get_scans> command.
         """
@@ -490,7 +491,7 @@ class StartScan(BaseCommand):
         return elements
 
     def handle_xml(self, xml: Element) -> bytes:
-        """ Handles <start_scan> command.
+        """Handles <start_scan> command.
 
         Return:
             Response string for <start_scan> command.
