@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -34,7 +34,7 @@ from ospd.server import TlsServer, UnixSocketServer, BaseServer
 from ospd.logger import init_logging
 
 
-COPYRIGHT = """Copyright (C) 2014-2020 Greenbone Networks GmbH
+COPYRIGHT = """Copyright (C) 2014-2021 Greenbone Networks GmbH
 License GPLv2+: GNU GPL version 2 or later
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law."""
@@ -109,7 +109,7 @@ def main(
 
     if args.port == 0:
         server = UnixSocketServer(
-            args.unix_socket, args.socket_mode, args.stream_timeout,
+            args.unix_socket, args.socket_mode, args.stream_timeout
         )
     else:
         server = TlsServer(
