@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -27,47 +27,34 @@ import multiprocessing
 import time
 import os
 
-from typing import (
-    List,
-    Any,
-    Iterator,
-    Dict,
-    Optional,
-    Iterable,
-    Tuple,
-    Union,
-)
+from pprint import pformat
+from typing import List, Any, Iterator, Dict, Optional, Iterable, Tuple, Union
 from xml.etree.ElementTree import Element, SubElement
 
 import defusedxml.ElementTree as secET
 
 import psutil
 
-from deprecated import deprecated
-
 from ospd import __version__
 from ospd.command import get_commands
 from ospd.errors import OspdCommandError
 from ospd.misc import ResultType, create_process
 from ospd.network import resolve_hostname, target_str_to_list
-from ospd.protocol import OspRequest, OspResponse, RequestParser
+from ospd.protocol import RequestParser
 from ospd.scan import ScanCollection, ScanStatus, ScanProgress
 from ospd.server import BaseServer, Stream
 from ospd.vtfilter import VtsFilter
 from ospd.vts import Vts
-from ospd.xml import (
-    elements_as_text,
-    get_result_xml,
-    get_progress_xml,
-    get_elements_from_dict,
-)
+from ospd.xml import elements_as_text, get_result_xml, get_progress_xml
 
 logger = logging.getLogger(__name__)
 
-PROTOCOL_VERSION = "1.2"
+PROTOCOL_VERSION = __version__
 
 SCHEDULER_CHECK_PERIOD = 10  # in seconds
 
+MIN_TIME_BETWEEN_START_SCAN = 60  # in seconds
+
 BASE_SCANNER_PARAMS = {
     'debug_mode': {
         'type': 'boolean',
@@ -92,7 +79,7 @@ def _terminate_process_group(process: multiprocessing.Process) -> None:
 
 class OSPDaemon:
 
-    """ Daemon class for OSP traffic handling.
+    """Daemon class for OSP traffic handling.
 
     Every scanner wrapper should subclass it and make necessary additions and
     changes.
@@ -118,7 +105,7 @@ class OSPDaemon:
         min_free_mem_scan_queue=0,
         file_storage_dir='/var/run/ospd',
         max_queued_scans=0,
-        **kwargs
+        **kwargs,
     ):  # pylint: disable=unused-argument
         """ Initializes the daemon's internal data. """
         self.scan_collection = ScanCollection(file_storage_dir)
@@ -141,6 +128,7 @@ class OSPDaemon:
         self.max_scans = max_scans
         self.min_free_mem_scan_queue = min_free_mem_scan_queue
         self.max_queued_scans = max_queued_scans
+        self.last_scan_start_time = 0
 
         self.scaninfo_store_time = kwargs.get('scaninfo_store_time')
 
@@ -166,9 +154,9 @@ class OSPDaemon:
             self.vts_filter = VtsFilter()
 
     def init(self, server: BaseServer) -> None:
-        """ Should be overridden by a subclass if the initialization is costly.
+        """Should be overridden by a subclass if the initialization is costly.
 
-            Will be called after check.
+        Will be called after check.
         """
         self.scan_collection.init()
         server.start(self.handle_client_stream)
@@ -180,11 +168,6 @@ class OSPDaemon:
             command = self.commands.get(name)
             command.attributes = attributes
 
-    @deprecated(version="20.8", reason="Use set_scanner_param instead")
-    def add_scanner_param(self, name: str, scanner_params: Dict) -> None:
-        """ Set a scanner parameter. """
-        self.set_scanner_param(name, scanner_params)
-
     def set_scanner_param(self, name: str, scanner_params: Dict) -> None:
         """ Set a scanner parameter. """
 
@@ -218,7 +201,7 @@ class OSPDaemon:
         qod_v: str = None,
         severities: str = None,
     ) -> None:
-        """ Add a vulnerability test information.
+        """Add a vulnerability test information.
 
         IMPORTANT: The VT's Data Manager will store the vts collection.
         If the collection is considerably big and it will be consultated
@@ -250,7 +233,7 @@ class OSPDaemon:
         )
 
     def set_vts_version(self, vts_version: str) -> None:
-        """ Add into the vts dictionary an entry to identify the
+        """Add into the vts dictionary an entry to identify the
         vts version.
 
         Parameters:
@@ -263,8 +246,7 @@ class OSPDaemon:
         self.vts_version = vts_version
 
     def get_vts_version(self) -> Optional[str]:
-        """Return the vts version.
-        """
+        """Return the vts version."""
         return self.vts_version
 
     def command_exists(self, name: str) -> bool:
@@ -311,7 +293,7 @@ class OSPDaemon:
                 except ValueError:
                     raise OspdCommandError(
                         'Invalid %s value' % key, 'start_scan'
-                    )
+                    ) from None
 
             if param_type == 'boolean':
                 if params[key] not in [0, 1]:
@@ -332,34 +314,9 @@ class OSPDaemon:
         return params
 
     def process_scan_params(self, params: Dict) -> Dict:
-        """ This method is to be overridden by the child classes if necessary
-        """
+        """This method is to be overridden by the child classes if necessary"""
         return params
 
-    @staticmethod
-    @deprecated(
-        version="20.8",
-        reason="Please use OspRequest.process_vt_params instead.",
-    )
-    def process_vts_params(scanner_vts) -> Dict:
-        return OspRequest.process_vts_params(scanner_vts)
-
-    @staticmethod
-    @deprecated(
-        version="20.8",
-        reason="Please use OspRequest.process_credential_elements instead.",
-    )
-    def process_credentials_elements(cred_tree) -> Dict:
-        return OspRequest.process_credentials_elements(cred_tree)
-
-    @staticmethod
-    @deprecated(
-        version="20.8",
-        reason="Please use OspRequest.process_targets_elements instead.",
-    )
-    def process_targets_element(scanner_target) -> List:
-        return OspRequest.process_target_element(scanner_target)
-
     def stop_scan(self, scan_id: str) -> None:
         if (
             scan_id in self.scan_collection.ids_iterator()
@@ -387,14 +344,15 @@ class OSPDaemon:
             '%s: Stopping Scan with the PID %s.', scan_id, scan_process.ident
         )
 
-        self.stop_scan_cleanup(scan_id)
-
         try:
             scan_process.terminate()
         except AttributeError:
             logger.debug('%s: The scanner task stopped unexpectedly.', scan_id)
 
         try:
+            logger.debug(
+                '%s: Terminating process group after stopping.', scan_id
+            )
             _terminate_process_group(scan_process)
         except ProcessLookupError:
             logger.info(
@@ -408,16 +366,6 @@ class OSPDaemon:
 
         logger.info('%s: Scan stopped.', scan_id)
 
-    @staticmethod
-    def stop_scan_cleanup(scan_id: str):
-        """ Should be implemented by subclass in case of a clean up before
-        terminating is needed. """
-
-    @staticmethod
-    def target_is_finished(scan_id: str):
-        """ Should be implemented by subclass in case of a check before
-        stopping is needed. """
-
     def exec_scan(self, scan_id: str):
         """ Asserts to False. Should be implemented by subclass. """
         raise NotImplementedError
@@ -445,6 +393,7 @@ class OSPDaemon:
                 and status != ScanStatus.FINISHED
                 and status != ScanStatus.INTERRUPTED
             ):
+                logger.debug("%s: Stopping scan before daemon exit.", scan_id)
                 self.stop_scan(scan_id)
 
         # Wait for scans to be in some stopped state.
@@ -460,8 +409,12 @@ class OSPDaemon:
                     all_stopped = False
 
             if all_stopped:
+                logger.debug(
+                    "All scans stopped and daemon clean and ready to exit"
+                )
                 return
 
+            logger.debug("Waiting for running scans before daemon exit. ")
             time.sleep(1)
 
     def get_daemon_name(self) -> str:
@@ -496,14 +449,6 @@ class OSPDaemon:
             return None
         return entry.get('default')
 
-    @deprecated(
-        version="20.8",
-        reason="Please use OspResponse.create_scanner_params_xml instead.",
-    )
-    def get_scanner_params_xml(self):
-        """ Returns the OSP Daemon's scanner params in xml format. """
-        return OspResponse.create_scanner_params_xml(self.scanner_params)
-
     def handle_client_stream(self, stream: Stream) -> None:
         """ Handles stream of data received from client. """
         data = b''
@@ -576,19 +521,28 @@ class OSPDaemon:
                 host=self.get_scan_host(scan_id),
                 value='Host process failure (%s).' % e,
             )
-            logger.exception('While scanning: %s', scan_id)
+            logger.exception('%s: Exception %s while scanning', scan_id, e)
         else:
             logger.info("%s: Host scan finished.", scan_id)
 
-        is_stopped = self.get_scan_status(scan_id) == ScanStatus.STOPPED
+        status = self.get_scan_status(scan_id)
+        is_stopped = status == ScanStatus.STOPPED
         self.set_scan_progress(scan_id)
         progress = self.get_scan_progress(scan_id)
-
         if not is_stopped and progress == ScanProgress.FINISHED:
             self.finish_scan(scan_id)
         elif not is_stopped:
+            logger.info(
+                "%s: Host scan got interrupted. Progress: %d, Status: %s",
+                scan_id,
+                progress,
+                status.name,
+            )
             self.interrupt_scan(scan_id)
 
+        # For debug purposes
+        self._get_scan_progress_raw(scan_id)
+
     def dry_run_scan(self, scan_id: str, target: Dict) -> None:
         """ Dry runs a scan. """
 
@@ -616,10 +570,10 @@ class OSPDaemon:
         )
 
     def sort_host_finished(
-        self, scan_id: str, finished_hosts: Union[List[str], str],
+        self, scan_id: str, finished_hosts: Union[List[str], str]
     ) -> None:
-        """ Check if the finished host in the list was alive or dead
-        and update the corresponding alive_count or dead_count. """
+        """Check if the finished host in the list was alive or dead
+        and update the corresponding alive_count or dead_count."""
         if isinstance(finished_hosts, str):
             finished_hosts = [finished_hosts]
 
@@ -633,8 +587,16 @@ class OSPDaemon:
             progress = current_hosts.get(finished_host)
             if progress == ScanProgress.FINISHED:
                 alive_hosts.append(finished_host)
-            if progress == ScanProgress.DEAD_HOST:
+            elif progress == ScanProgress.DEAD_HOST:
                 dead_hosts.append(finished_host)
+            else:
+                logger.debug(
+                    'The host %s is considered dead or finished, but '
+                    'its progress is still %d. This can lead to '
+                    'interrupted scan.',
+                    finished_host,
+                    progress,
+                )
 
         self.scan_collection.set_host_dead(scan_id, dead_hosts)
 
@@ -645,8 +607,12 @@ class OSPDaemon:
         )
 
     def set_scan_progress(self, scan_id: str):
-        """ Calculate the target progress with the current host states
-        and stores in the scan table. """
+        """Calculate the target progress with the current host states
+        and stores in the scan table."""
+        # Get current scan progress for debugging purposes
+        logger.debug("Calculating scan progress with the following data:")
+        self._get_scan_progress_raw(scan_id)
+
         scan_progress = self.scan_collection.calculate_target_progress(scan_id)
         self.scan_collection.set_progress(scan_id, scan_progress)
 
@@ -657,9 +623,9 @@ class OSPDaemon:
         self.set_scan_progress(scan_id)
 
     def set_scan_host_progress(
-        self, scan_id: str, host: str = None, progress: int = None,
+        self, scan_id: str, host: str = None, progress: int = None
     ) -> None:
-        """ Sets host's progress which is part of target.
+        """Sets host's progress which is part of target.
         Each time a host progress is updated, the scan progress
         is updated too.
         """
@@ -675,18 +641,29 @@ class OSPDaemon:
         host_progress = {host: progress}
         self.set_scan_progress_batch(scan_id, host_progress)
 
+    def get_scan_host_progress(self, scan_id: str, host: str = None) -> int:
+        """ Get host's progress which is part of target."""
+        current_progress = self.scan_collection.get_current_target_progress(
+            scan_id
+        )
+        return current_progress.get(host)
+
     def set_scan_status(self, scan_id: str, status: ScanStatus) -> None:
         """ Set the scan's status."""
+        logger.debug('%s: Set scan status %s,', scan_id, status.name)
         self.scan_collection.set_status(scan_id, status)
 
     def get_scan_status(self, scan_id: str) -> ScanStatus:
         """ Get scan_id scans's status."""
-        return self.scan_collection.get_status(scan_id)
+        status = self.scan_collection.get_status(scan_id)
+        logger.debug('%s: Current scan status: %s,', scan_id, status.name)
+        return status
 
     def scan_exists(self, scan_id: str) -> bool:
-        """ Checks if a scan with ID scan_id is in collection.
+        """Checks if a scan with ID scan_id is in collection.
 
-        @return: 1 if scan exists, 0 otherwise.
+        Returns:
+            1 if scan exists, 0 otherwise.
         """
         return self.scan_collection.id_exists(scan_id)
 
@@ -710,22 +687,18 @@ class OSPDaemon:
 
             if elements:
                 command_txt = ''.join(
-                    [command_txt, "\t Elements:\n", elements_as_text(elements),]
+                    [command_txt, "\t Elements:\n", elements_as_text(elements)]
                 )
 
             txt += command_txt
 
         return txt
 
-    @deprecated(version="20.8", reason="Use ospd.xml.elements_as_text instead.")
-    def elements_as_text(self, elems: Dict, indent: int = 2) -> str:
-        """ Returns the elems dictionary as formatted plain text. """
-        return elements_as_text(elems, indent)
-
     def delete_scan(self, scan_id: str) -> int:
-        """ Deletes scan_id scan from collection.
+        """Deletes scan_id scan from collection.
 
-        @return: 1 if scan deleted, 0 otherwise.
+        Returns:
+            1 if scan deleted, 0 otherwise.
         """
         if self.get_scan_status(scan_id) == ScanStatus.RUNNING:
             return 0
@@ -746,9 +719,10 @@ class OSPDaemon:
     def get_scan_results_xml(
         self, scan_id: str, pop_res: bool, max_res: Optional[int]
     ):
-        """ Gets scan_id scan's results in XML format.
+        """Gets scan_id scan's results in XML format.
 
-        @return: String of scan results in xml.
+        Returns:
+            String of scan results in xml.
         """
         results = Element('results')
         for result in self.scan_collection.results_iterator(
@@ -759,11 +733,8 @@ class OSPDaemon:
         logger.debug('Returning %d results', len(results))
         return results
 
-    def _get_scan_progress_xml(self, scan_id: str):
-        """ Gets scan_id scan's progress in XML format.
-
-        @return: String of scan progress in xml.
-        """
+    def _get_scan_progress_raw(self, scan_id: str) -> Dict:
+        """Returns a dictionary with scan_id scan's progress information."""
         current_progress = dict()
 
         current_progress[
@@ -778,25 +749,24 @@ class OSPDaemon:
         )
         current_progress[
             'count_excluded'
-        ] = self.scan_collection.simplify_exclude_host_count(scan_id)
-        current_progress['count_total'] = self.scan_collection.get_host_count(
+        ] = self.scan_collection.get_simplified_exclude_host_count(scan_id)
+        current_progress['count_total'] = self.scan_collection.get_count_total(
             scan_id
         )
 
-        return get_progress_xml(current_progress)
+        logging.debug(
+            "%s: Current progress: \n%s", scan_id, pformat(current_progress)
+        )
+        return current_progress
 
-    @deprecated(
-        version="20.8",
-        reason="Please use ospd.xml.get_elements_from_dict instead.",
-    )
-    def get_xml_str(self, data: Dict) -> List:
-        """ Creates a string in XML Format using the provided data structure.
+    def _get_scan_progress_xml(self, scan_id: str):
+        """Gets scan_id scan's progress in XML format.
 
-        @param: Dictionary of xml tags and their elements.
-
-        @return: String of data in xml format.
+        Returns:
+            String of scan progress in xml.
         """
-        return get_elements_from_dict(data)
+        current_progress = self._get_scan_progress_raw(scan_id)
+        return get_progress_xml(current_progress)
 
     def get_scan_xml(
         self,
@@ -806,9 +776,10 @@ class OSPDaemon:
         max_res: int = 0,
         progress: bool = False,
     ):
-        """ Gets scan in XML format.
+        """Gets scan in XML format.
 
-        @return: String of scan in XML format.
+        Returns:
+            String of scan in XML format.
         """
         if not scan_id:
             return Element('scan')
@@ -853,7 +824,7 @@ class OSPDaemon:
     def get_custom_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, custom: Dict
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         custom data object.
         This needs to be implemented by each ospd wrapper, in case
         custom elements for VTs are used.
@@ -861,7 +832,8 @@ class OSPDaemon:
         The custom XML object which is returned will be embedded
         into a <custom></custom> element.
 
-        @return: XML object as string for custom data.
+        Returns:
+            XML object as string for custom data.
         """
         return ''
 
@@ -869,7 +841,7 @@ class OSPDaemon:
     def get_params_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, vt_params
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         vt_params data object.
         This needs to be implemented by each ospd wrapper, in case
         vt_params elements for VTs are used.
@@ -877,7 +849,8 @@ class OSPDaemon:
         The params XML object which is returned will be embedded
         into a <params></params> element.
 
-        @return: XML object as string for vt parameters data.
+        Returns:
+            XML object as string for vt parameters data.
         """
         return ''
 
@@ -885,7 +858,7 @@ class OSPDaemon:
     def get_refs_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, vt_refs
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         refs data object.
         This needs to be implemented by each ospd wrapper, in case
         refs elements for VTs are used.
@@ -893,7 +866,8 @@ class OSPDaemon:
         The refs XML object which is returned will be embedded
         into a <refs></refs> element.
 
-        @return: XML object as string for vt references data.
+        Returns:
+            XML object as string for vt references data.
         """
         return ''
 
@@ -901,7 +875,7 @@ class OSPDaemon:
     def get_dependencies_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, vt_dependencies
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         vt_dependencies data object.
         This needs to be implemented by each ospd wrapper, in case
         vt_dependencies elements for VTs are used.
@@ -909,7 +883,8 @@ class OSPDaemon:
         The vt_dependencies XML object which is returned will be embedded
         into a <dependencies></dependencies> element.
 
-        @return: XML object as string for vt dependencies data.
+        Returns:
+            XML object as string for vt dependencies data.
         """
         return ''
 
@@ -917,7 +892,7 @@ class OSPDaemon:
     def get_creation_time_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, vt_creation_time
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         vt_creation_time data object.
         This needs to be implemented by each ospd wrapper, in case
         vt_creation_time elements for VTs are used.
@@ -925,7 +900,8 @@ class OSPDaemon:
         The vt_creation_time XML object which is returned will be embedded
         into a <vt_creation_time></vt_creation_time> element.
 
-        @return: XML object as string for vt creation time data.
+        Returns:
+            XML object as string for vt creation time data.
         """
         return ''
 
@@ -933,7 +909,7 @@ class OSPDaemon:
     def get_modification_time_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, vt_modification_time
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         vt_modification_time data object.
         This needs to be implemented by each ospd wrapper, in case
         vt_modification_time elements for VTs are used.
@@ -941,7 +917,8 @@ class OSPDaemon:
         The vt_modification_time XML object which is returned will be embedded
         into a <vt_modification_time></vt_modification_time> element.
 
-        @return: XML object as string for vt references data.
+        Returns:
+            XML object as string for vt references data.
         """
         return ''
 
@@ -949,7 +926,7 @@ class OSPDaemon:
     def get_summary_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, summary
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         summary data object.
         This needs to be implemented by each ospd wrapper, in case
         summary elements for VTs are used.
@@ -957,7 +934,8 @@ class OSPDaemon:
         The summary XML object which is returned will be embedded
         into a <summary></summary> element.
 
-        @return: XML object as string for summary data.
+        Returns:
+            XML object as string for summary data.
         """
         return ''
 
@@ -965,7 +943,7 @@ class OSPDaemon:
     def get_impact_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, impact
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         impact data object.
         This needs to be implemented by each ospd wrapper, in case
         impact elements for VTs are used.
@@ -973,7 +951,8 @@ class OSPDaemon:
         The impact XML object which is returned will be embedded
         into a <impact></impact> element.
 
-        @return: XML object as string for impact data.
+        Returns:
+            XML object as string for impact data.
         """
         return ''
 
@@ -981,7 +960,7 @@ class OSPDaemon:
     def get_affected_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, affected
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         affected data object.
         This needs to be implemented by each ospd wrapper, in case
         affected elements for VTs are used.
@@ -989,7 +968,8 @@ class OSPDaemon:
         The affected XML object which is returned will be embedded
         into a <affected></affected> element.
 
-        @return: XML object as string for affected data.
+        Returns:
+            XML object as string for affected data.
         """
         return ''
 
@@ -997,7 +977,7 @@ class OSPDaemon:
     def get_insight_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, insight
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         insight data object.
         This needs to be implemented by each ospd wrapper, in case
         insight elements for VTs are used.
@@ -1005,7 +985,8 @@ class OSPDaemon:
         The insight XML object which is returned will be embedded
         into a <insight></insight> element.
 
-        @return: XML object as string for insight data.
+        Returns:
+            XML object as string for insight data.
         """
         return ''
 
@@ -1013,7 +994,7 @@ class OSPDaemon:
     def get_solution_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, solution, solution_type=None, solution_method=None
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         solution data object.
         This needs to be implemented by each ospd wrapper, in case
         solution elements for VTs are used.
@@ -1021,7 +1002,8 @@ class OSPDaemon:
         The solution XML object which is returned will be embedded
         into a <solution></solution> element.
 
-        @return: XML object as string for solution data.
+        Returns:
+            XML object as string for solution data.
         """
         return ''
 
@@ -1029,7 +1011,7 @@ class OSPDaemon:
     def get_detection_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, detection=None, qod_type=None, qod=None
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         detection data object.
         This needs to be implemented by each ospd wrapper, in case
         detection elements for VTs are used.
@@ -1037,7 +1019,8 @@ class OSPDaemon:
         The detection XML object which is returned is an element with
         tag <detection></detection> element
 
-        @return: XML object as string for detection data.
+        Returns:
+            XML object as string for detection data.
         """
         return ''
 
@@ -1045,7 +1028,7 @@ class OSPDaemon:
     def get_severities_vt_as_xml_str(  # pylint: disable=unused-argument
         vt_id: str, severities
     ) -> str:
-        """ Create a string representation of the XML object from the
+        """Create a string representation of the XML object from the
         severities data object.
         This needs to be implemented by each ospd wrapper, in case
         severities elements for VTs are used.
@@ -1053,21 +1036,23 @@ class OSPDaemon:
         The severities XML objects which are returned will be embedded
         into a <severities></severities> element.
 
-        @return: XML object as string for severities data.
+        Returns:
+            XML object as string for severities data.
         """
         return ''
 
     def get_vt_iterator(  # pylint: disable=unused-argument
         self, vt_selection: List[str] = None, details: bool = True
     ) -> Iterator[Tuple[str, Dict]]:
-        """ Return iterator object for getting elements
-        from the VTs dictionary. """
+        """Return iterator object for getting elements
+        from the VTs dictionary."""
         return self.vts.items()
 
     def get_vt_xml(self, single_vt: Tuple[str, Dict]) -> Element:
-        """ Gets a single vulnerability test information in XML format.
+        """Gets a single vulnerability test information in XML format.
 
-        @return: String of single vulnerability test information in XML format.
+        Returns:
+            String of single vulnerability test information in XML format.
         """
         if not single_vt or single_vt[1] is None:
             return Element('vt')
@@ -1179,7 +1164,7 @@ class OSPDaemon:
             vt_id (vt_id, optional): ID of the vt to get.
             filtered_vts (list, optional): Filtered VTs collection.
 
-        Return:
+        Returns:
             List of selected VT's OID.
         """
         vts_xml = []
@@ -1198,13 +1183,12 @@ class OSPDaemon:
         return vts_list
 
     def handle_command(self, data: bytes, stream: Stream) -> None:
-        """ Handles an osp command in a string.
-        """
+        """Handles an osp command in a string."""
         try:
             tree = secET.fromstring(data)
-        except secET.ParseError:
+        except secET.ParseError as e:
             logger.debug("Erroneous client input: %s", data)
-            raise OspdCommandError('Invalid data')
+            raise OspdCommandError('Invalid data') from e
 
         command_name = tree.tag
 
@@ -1236,8 +1220,18 @@ class OSPDaemon:
         scan_id = tree.get('scan_id')
         if self.scan_exists(scan_id) and command_name == "get_scans":
             if write_success:
+                logger.debug(
+                    '%s: Results sent successfully to the client. Cleaning '
+                    'temporary result list.',
+                    scan_id,
+                )
                 self.scan_collection.clean_temp_result_list(scan_id)
             else:
+                logger.debug(
+                    '%s: Failed sending results to the client. Restoring '
+                    'result list into the cache.',
+                    scan_id,
+                )
                 self.scan_collection.restore_temp_result_list(scan_id)
 
     def check(self):
@@ -1245,8 +1239,7 @@ class OSPDaemon:
         raise NotImplementedError
 
     def run(self) -> None:
-        """ Starts the Daemon, handling commands until interrupted.
-        """
+        """Starts the Daemon, handling commands until interrupted."""
 
         try:
             while True:
@@ -1295,14 +1288,15 @@ class OSPDaemon:
                 self.set_scan_status(scan_id, ScanStatus.INIT)
 
                 current_queued_scans = current_queued_scans - 1
+                self.last_scan_start_time = time.time()
                 logger.info('Starting scan %s.', scan_id)
             elif scan_is_queued and not scan_allowed:
                 return
 
     def is_new_scan_allowed(self) -> bool:
-        """ Check if max_scans has been reached.
+        """Check if max_scans has been reached.
 
-        Return:
+        Returns:
             True if a new scan can be launch.
         """
         if (self.max_scans != 0) and (
@@ -1318,16 +1312,30 @@ class OSPDaemon:
         return True
 
     def is_enough_free_memory(self) -> bool:
-        """ Check if there is enough free memory in the system to run
+        """Check if there is enough free memory in the system to run
         a new scan. The necessary memory is a rough calculation and very
         conservative.
 
-        Return:
+        Returns:
             True if there is enough memory for a new scan.
         """
         if not self.min_free_mem_scan_queue:
             return True
 
+        # If min_free_mem_scan_queue option is set, also wait some time
+        # between scans. Consider the case in which the last scan
+        # finished in a few seconds and there is no need to wait.
+        time_between_start_scan = time.time() - self.last_scan_start_time
+        if (
+            time_between_start_scan < MIN_TIME_BETWEEN_START_SCAN
+            and self.get_count_running_scans()
+        ):
+            logger.debug(
+                'Not possible to run a new scan right now, a scan have been '
+                'just started.'
+            )
+            return False
+
         free_mem = psutil.virtual_memory().available / (1024 * 1024)
 
         if free_mem > self.min_free_mem_scan_queue:
@@ -1343,12 +1351,12 @@ class OSPDaemon:
         return False
 
     def scheduler(self):
-        """ Should be implemented by subclass in case of need
-        to run tasks periodically. """
+        """Should be implemented by subclass in case of need
+        to run tasks periodically."""
 
     def wait_for_children(self):
         """ Join the zombie process to releases resources."""
-        for scan_id in self.scan_processes:
+        for scan_id, _ in self.scan_processes.items():
             self.scan_processes[scan_id].join(0)
 
     def create_scan(
@@ -1358,13 +1366,15 @@ class OSPDaemon:
         options: Optional[Dict],
         vt_selection: Dict,
     ) -> Optional[str]:
-        """ Creates a new scan.
+        """Creates a new scan.
 
-        @target: Target to scan.
-        @options: Miscellaneous scan options supplied via <scanner_params>
+        Arguments:
+            target: Target to scan.
+            options: Miscellaneous scan options supplied via <scanner_params>
                   XML element.
 
-        @return: New scan's ID. None if the scan_id already exists.
+        Returns:
+            New scan's ID. None if the scan_id already exists.
         """
         status = None
         scan_exists = self.scan_exists(scan_id)
@@ -1387,8 +1397,13 @@ class OSPDaemon:
         """ Sets a scan's option to a provided value. """
         return self.scan_collection.set_option(scan_id, name, value)
 
+    def set_scan_total_hosts(self, scan_id: str, count_total: int) -> None:
+        """Sets a scan's total hosts. Allow the scanner to update
+        the total count of host to be scanned."""
+        self.scan_collection.update_count_total(scan_id, count_total)
+
     def clean_forgotten_scans(self) -> None:
-        """ Check for old stopped or finished scans which have not been
+        """Check for old stopped or finished scans which have not been
         deleted and delete them if the are older than the set value."""
 
         if not self.scaninfo_store_time:
@@ -1416,7 +1431,8 @@ class OSPDaemon:
 
     def check_scan_process(self, scan_id: str) -> None:
         """ Check the scan's process, and terminate the scan if not alive. """
-        if self.get_scan_status(scan_id) == ScanStatus.QUEUED:
+        status = self.get_scan_status(scan_id)
+        if status == ScanStatus.QUEUED:
             return
 
         scan_process = self.scan_processes.get(scan_id)
@@ -1427,18 +1443,28 @@ class OSPDaemon:
             and scan_process
             and not scan_process.is_alive()
         ):
-            if not self.get_scan_status(scan_id) == ScanStatus.STOPPED:
-                self.set_scan_status(scan_id, ScanStatus.STOPPED)
+            if not status == ScanStatus.STOPPED:
                 self.add_scan_error(
-                    scan_id, name="", host="", value="Scan process failure."
+                    scan_id, name="", host="", value="Scan process Failure"
                 )
 
-                logger.info("%s: Scan stopped with errors.", scan_id)
+                logger.info(
+                    "%s: Scan process is dead and its progress is %d",
+                    scan_id,
+                    progress,
+                )
                 self.interrupt_scan(scan_id)
 
         elif progress == ScanProgress.FINISHED:
             scan_process.join(0)
 
+        logger.debug(
+            "%s: Check scan process: \n\tProgress %d\n\t Status: %s",
+            scan_id,
+            progress,
+            status.name,
+        )
+
     def get_count_queued_scans(self) -> int:
         """ Get the amount of scans with queued status """
         count = 0
@@ -1447,9 +1473,20 @@ class OSPDaemon:
                 count += 1
         return count
 
+    def get_count_running_scans(self) -> int:
+        """ Get the amount of scans with INIT/RUNNING status """
+        count = 0
+        for scan_id in self.scan_collection.ids_iterator():
+            status = self.get_scan_status(scan_id)
+            if status == ScanStatus.RUNNING or status == ScanStatus.INIT:
+                count += 1
+        return count
+
     def get_scan_progress(self, scan_id: str) -> int:
         """ Gives a scan's current progress value. """
-        return self.scan_collection.get_progress(scan_id)
+        progress = self.scan_collection.get_progress(scan_id)
+        logger.debug('%s: Current scan progress: %s,', scan_id, progress)
+        return progress
 
     def get_scan_host(self, scan_id: str) -> str:
         """ Gives a scan's target. """
@@ -1460,18 +1497,18 @@ class OSPDaemon:
         return self.scan_collection.get_ports(scan_id)
 
     def get_scan_exclude_hosts(self, scan_id: str):
-        """ Gives a scan's exclude host list. If a target is passed gives
-        the exclude host list for the given target. """
+        """Gives a scan's exclude host list. If a target is passed gives
+        the exclude host list for the given target."""
         return self.scan_collection.get_exclude_hosts(scan_id)
 
     def get_scan_credentials(self, scan_id: str) -> Dict:
-        """ Gives a scan's credential list. If a target is passed gives
-        the credential list for the given target. """
+        """Gives a scan's credential list. If a target is passed gives
+        the credential list for the given target."""
         return self.scan_collection.get_credentials(scan_id)
 
     def get_scan_target_options(self, scan_id: str) -> Dict:
-        """ Gives a scan's target option dict. If a target is passed gives
-        the credential list for the given target. """
+        """Gives a scan's target option dict. If a target is passed gives
+        the credential list for the given target."""
         return self.scan_collection.get_target_options(scan_id)
 
     def get_scan_vts(self, scan_id: str) -> Dict:
