@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -20,6 +20,7 @@ import multiprocessing
 import time
 import uuid
 
+from pprint import pformat
 from collections import OrderedDict
 from enum import Enum, IntEnum
 from typing import List, Any, Dict, Iterator, Optional, Iterable, Union
@@ -53,7 +54,7 @@ class ScanProgress(IntEnum):
 
 class ScanCollection:
 
-    """ Scans collection, managing scans and results read and write, exposing
+    """Scans collection, managing scans and results read and write, exposing
     only needed information.
 
     Each scan has meta-information such as scan ID, current progress (from 0 to
@@ -137,6 +138,13 @@ class ScanCollection:
         if not hosts:
             return
 
+        LOGGER.debug(
+            '%s: Remove the following hosts from the target list, '
+            'as they are already finished or are dead: %s',
+            scan_id,
+            pformat(hosts),
+        )
+
         target = self.scans_table[scan_id].get('target_progress')
         for host in hosts:
             if host in target:
@@ -170,6 +178,11 @@ class ScanCollection:
     def set_host_finished(self, scan_id: str, hosts: List[str]) -> None:
         """ Increase the amount of finished hosts which were alive."""
 
+        LOGGER.debug(
+            '%s: Setting the following hosts as finished: %s',
+            scan_id,
+            pformat(hosts),
+        )
         total_finished = len(hosts)
         count_alive = (
             self.scans_table[scan_id].get('count_alive') + total_finished
@@ -179,6 +192,11 @@ class ScanCollection:
     def set_host_dead(self, scan_id: str, hosts: List[str]) -> None:
         """ Increase the amount of dead hosts. """
 
+        LOGGER.debug(
+            '%s: Setting the following hosts as dead: %s',
+            scan_id,
+            pformat(hosts),
+        )
         total_dead = len(hosts)
         count_dead = self.scans_table[scan_id].get('count_dead') + total_dead
         self.scans_table[scan_id]['count_dead'] = count_dead
@@ -194,8 +212,8 @@ class ScanCollection:
         self.scans_table[scan_id]['temp_results'] = list()
 
     def restore_temp_result_list(self, scan_id):
-        """ Add the results stored in the temporary list into the results
-        list again. """
+        """Add the results stored in the temporary list into the results
+        list again."""
         result_aux = self.scans_table[scan_id].get('results', list())
         result_aux.extend(self.scans_table[scan_id].get('temp_results', list()))
 
@@ -206,7 +224,7 @@ class ScanCollection:
     def results_iterator(
         self, scan_id: str, pop_res: bool = False, max_res: int = None
     ) -> Iterator[Any]:
-        """ Returns an iterator over scan_id scan's results. If pop_res is True,
+        """Returns an iterator over scan_id scan's results. If pop_res is True,
         it removed the fetched results from the list.
 
         If max_res is None, return all the results.
@@ -247,8 +265,8 @@ class ScanCollection:
         pickler.remove_file(scan_id)
 
     def unpickle_scan_info(self, scan_id: str) -> None:
-        """ Unpickle a stored scan_inf corresponding to the scan_id
-        and store it in the scan_table """
+        """Unpickle a stored scan_inf corresponding to the scan_id
+        and store it in the scan_table"""
 
         scan_info = self.scans_table.get(scan_id)
         scan_info_hash = scan_info.pop('scan_info_hash')
@@ -269,6 +287,8 @@ class ScanCollection:
         scan_info['target_progress'] = dict()
         scan_info['count_alive'] = 0
         scan_info['count_dead'] = 0
+        scan_info['count_total'] = None
+        scan_info['excluded_simplified'] = None
         scan_info['target'] = unpickled_scan_info.pop('target')
         scan_info['vts'] = unpickled_scan_info.pop('vts')
         scan_info['options'] = unpickled_scan_info.pop('options')
@@ -286,7 +306,7 @@ class ScanCollection:
         options: Optional[Dict] = None,
         vts: Dict = None,
     ) -> str:
-        """ Creates a new scan with provided scan information.
+        """Creates a new scan with provided scan information.
 
         @target: Target to scan.
         @options: Miscellaneous scan options supplied via <scanner_params>
@@ -304,12 +324,9 @@ class ScanCollection:
         scan_info['status'] = ScanStatus.QUEUED
         scan_info['credentials'] = credentials
         scan_info['start_time'] = int(time.time())
+        scan_info['end_time'] = 0
 
-        scan_info_to_pickle = {
-            'target': target,
-            'options': options,
-            'vts': vts,
-        }
+        scan_info_to_pickle = {'target': target, 'options': options, 'vts': vts}
 
         if scan_id is None or scan_id == '':
             scan_id = str(uuid.uuid4())
@@ -331,7 +348,7 @@ class ScanCollection:
     def set_status(self, scan_id: str, status: ScanStatus) -> None:
         """ Sets scan_id scan's status. """
         self.scans_table[scan_id]['status'] = status
-        if status == ScanStatus.STOPPED:
+        if status == ScanStatus.STOPPED or status == ScanStatus.INTERRUPTED:
             self.scans_table[scan_id]['end_time'] = int(time.time())
 
     def get_status(self, scan_id: str) -> ScanStatus:
@@ -360,16 +377,42 @@ class ScanCollection:
         return self.scans_table[scan_id]['count_dead']
 
     def get_count_alive(self, scan_id: str) -> int:
-        """ Get a scan's current dead host count. """
+        """ Get a scan's current alive host count. """
 
         return self.scans_table[scan_id]['count_alive']
 
+    def update_count_total(self, scan_id: str, count_total: int) -> int:
+        """ Sets a scan's total hosts."""
+
+        self.scans_table[scan_id]['count_total'] = count_total
+
+    def get_count_total(self, scan_id: str) -> int:
+        """ Get a scan's total host count. """
+
+        count_total = self.scans_table[scan_id]['count_total']
+
+        # The value set by the server has priority over the value
+        # calculated from the original target list by ospd.
+        # As ospd is not intelligent enough to check the amount of valid
+        # hosts, check for duplicated or invalid hosts, consider a negative
+        # value set for the server, in case it detects an invalid target string
+        # or a different amount than the original amount in the target list.
+        if count_total == -1:
+            count_total = 0
+        # If the server does not set the total host count
+        # ospd set the amount of host from the original host list.
+        elif count_total is None:
+            count_total = self.get_host_count(scan_id)
+            self.update_count_total(scan_id, count_total)
+
+        return count_total
+
     def get_current_target_progress(self, scan_id: str) -> Dict[str, int]:
         """ Get a scan's current hosts progress """
         return self.scans_table[scan_id]['target_progress']
 
     def simplify_exclude_host_count(self, scan_id: str) -> int:
-        """ Remove from exclude_hosts the received hosts in the finished_hosts
+        """Remove from exclude_hosts the received hosts in the finished_hosts
         list sent by the client.
         The finished hosts are sent also as exclude hosts for backward
         compatibility purposses.
@@ -377,27 +420,59 @@ class ScanCollection:
         Return:
             Count of excluded host.
         """
-
         exc_hosts_list = target_str_to_list(self.get_exclude_hosts(scan_id))
 
         finished_hosts_list = target_str_to_list(
             self.get_finished_hosts(scan_id)
         )
 
+        # Remove finished hosts from excluded host list
         if finished_hosts_list and exc_hosts_list:
             for finished in finished_hosts_list:
                 if finished in exc_hosts_list:
                     exc_hosts_list.remove(finished)
 
-        return len(exc_hosts_list) if exc_hosts_list else 0
+        # Remove excluded hosts which don't belong to the target list
+        host_list = target_str_to_list(self.get_host_list(scan_id))
+        excluded_simplified = 0
+        invalid_exc_hosts = 0
+        if exc_hosts_list:
+            for exc_host in exc_hosts_list:
+                if exc_host in host_list:
+                    excluded_simplified += 1
+                else:
+                    invalid_exc_hosts += 1
+
+        if invalid_exc_hosts > 0:
+            LOGGER.warning(
+                "Please check the excluded host list. It contains hosts which "
+                "do not belong to the target. This warning can be ignored if "
+                "this was done on purpose (e.g. to exclude specific hostname)."
+            )
+
+        # Set scan_info's excluded simplified to propagate excluded count
+        # to parent process.
+        self.scans_table[scan_id]['excluded_simplified'] = excluded_simplified
+
+        return excluded_simplified
+
+    def get_simplified_exclude_host_count(self, scan_id: str) -> int:
+        """ Get a scan's excluded host count. """
+        excluded_simplified = self.scans_table[scan_id]['excluded_simplified']
+        # Check for None because it is the init value, as excluded can be 0
+        # as well
+        if excluded_simplified is not None:
+            return excluded_simplified
+
+        return self.simplify_exclude_host_count(scan_id)
 
     def calculate_target_progress(self, scan_id: str) -> int:
-        """ Get a target's current progress value.
+        """Get a target's current progress value.
         The value is calculated with the progress of each single host
         in the target."""
 
-        total_hosts = self.get_host_count(scan_id)
-        exc_hosts = self.simplify_exclude_host_count(scan_id)
+        total_hosts = self.get_count_total(scan_id)
+        exc_hosts = self.get_simplified_exclude_host_count(scan_id)
         count_alive = self.get_count_alive(scan_id)
         count_dead = self.get_count_dead(scan_id)
         host_progresses = self.get_current_target_progress(scan_id)
@@ -409,6 +484,7 @@ class ScanCollection:
             )
         except ZeroDivisionError:
             # Consider the case in which all hosts are dead or excluded
+            LOGGER.debug('%s: All hosts dead or excluded.', scan_id)
             t_prog = ScanProgress.FINISHED.value
 
         return t_prog
@@ -426,7 +502,17 @@ class ScanCollection:
     def get_host_list(self, scan_id: str) -> Dict:
         """ Get a scan's host list. """
 
-        return self.scans_table[scan_id]['target'].get('hosts')
+        target = None
+        try:
+            target = self.scans_table[scan_id]['target'].get('hosts')
+        except KeyError:
+            LOGGER.warning(
+                '%s: Scan ID is in the scan table, but it was '
+                'not initialized.',
+                scan_id,
+            )
+
+        return target
 
     def get_host_count(self, scan_id: str) -> int:
         """ Get total host count in the target. """
@@ -439,31 +525,28 @@ class ScanCollection:
         return total_hosts
 
     def get_ports(self, scan_id: str) -> str:
-        """ Get a scan's ports list.
-        """
+        """Get a scan's ports list."""
         target = self.scans_table[scan_id].get('target')
         ports = target.pop('ports')
         self.scans_table[scan_id]['target'] = target
         return ports
 
     def get_exclude_hosts(self, scan_id: str) -> str:
-        """ Get an exclude host list for a given target.
-        """
+        """Get an exclude host list for a given target."""
         return self.scans_table[scan_id]['target'].get('exclude_hosts')
 
     def get_finished_hosts(self, scan_id: str) -> str:
-        """ Get the finished host list sent by the client for a given target.
-        """
+        """Get the finished host list sent by the client for a given target."""
         return self.scans_table[scan_id]['target'].get('finished_hosts')
 
     def get_credentials(self, scan_id: str) -> Dict[str, Dict[str, str]]:
-        """ Get a scan's credential list. It return dictionary with
+        """Get a scan's credential list. It return dictionary with
         the corresponding credential for a given target.
         """
         return self.scans_table[scan_id].get('credentials')
 
     def get_target_options(self, scan_id: str) -> Dict[str, str]:
-        """ Get a scan's target option dictionary.
+        """Get a scan's target option dictionary.
         It return dictionary with the corresponding options for
         a given target.
         """
