@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -43,8 +43,7 @@ class Stream:
         self.socket.settimeout(stream_timeout)
 
     def close(self):
-        """ Close the stream
-        """
+        """Close the stream"""
         try:
             self.socket.shutdown(socket.SHUT_RDWR)
         except OSError as e:
@@ -55,8 +54,7 @@ class Stream:
         self.socket.close()
 
     def read(self, bufsize: Optional[int] = DEFAULT_BUFSIZE) -> bytes:
-        """ Read at maximum bufsize data from the stream
-        """
+        """Read at maximum bufsize data from the stream"""
         data = self.socket.recv(bufsize)
 
         if not data:
@@ -65,8 +63,7 @@ class Stream:
         return data
 
     def write(self, data: bytes) -> bool:
-        """ Send data in chunks of DEFAULT_BUFSIZE to the client
-        """
+        """Send data in chunks of DEFAULT_BUFSIZE to the client"""
         b_start = 0
         b_end = DEFAULT_BUFSIZE
         ret_success = True
@@ -106,7 +103,7 @@ def validate_cacert_file(cacert: str):
         # Python version < 2.7.9
         return
     except IOError:
-        raise OspdError('CA Certificate not found')
+        raise OspdError('CA Certificate not found') from None
 
     try:
         not_after = context.get_ca_certs()[0]['notAfter']
@@ -114,7 +111,7 @@ def validate_cacert_file(cacert: str):
         not_before = context.get_ca_certs()[0]['notBefore']
         not_before = ssl.cert_time_to_seconds(not_before)
     except (KeyError, IndexError):
-        raise OspdError('CA Certificate is erroneous')
+        raise OspdError('CA Certificate is erroneous') from None
 
     now = int(time.time())
     if not_after < now:
@@ -138,7 +135,7 @@ class BaseServer(ABC):
 
     @abstractmethod
     def start(self, stream_callback: StreamCallbackType):
-        """ Starts a server with capabilities to handle multiple client
+        """Starts a server with capabilities to handle multiple client
         connections simultaneously.
         If a new client connects the stream_callback is called with a Stream
 
@@ -183,20 +180,19 @@ class SocketServerMixin:
 
 
 class ThreadedUnixSocketServer(
-    SocketServerMixin, socketserver.ThreadingUnixStreamServer,
+    SocketServerMixin, socketserver.ThreadingUnixStreamServer
 ):
     pass
 
 
 class ThreadedTlsSocketServer(
-    SocketServerMixin, socketserver.ThreadingTCPServer,
+    SocketServerMixin, socketserver.ThreadingTCPServer
 ):
     pass
 
 
 class UnixSocketServer(BaseServer):
-    """ Server for accepting connections via a Unix domain socket
-    """
+    """Server for accepting connections via a Unix domain socket"""
 
     def __init__(self, socket_path: str, socket_mode: str, stream_timeout: int):
         super().__init__(stream_timeout)
@@ -226,7 +222,7 @@ class UnixSocketServer(BaseServer):
                 "Couldn't bind socket on {}. {}".format(
                     str(self.socket_path), e
                 )
-            )
+            ) from e
 
         if self.socket_path.exists():
             self.socket_path.chmod(self.socket_mode)
@@ -243,8 +239,7 @@ class UnixSocketServer(BaseServer):
 
 
 class TlsServer(BaseServer):
-    """ Server for accepting TLS encrypted connections via a TCP socket
-    """
+    """Server for accepting TLS encrypted connections via a TCP socket"""
 
     def __init__(
         self,
@@ -269,18 +264,7 @@ class TlsServer(BaseServer):
 
         validate_cacert_file(ca_file)
 
-        # Despite the name, ssl.PROTOCOL_SSLv23 selects the highest
-        # protocol version that both the client and server support. In modern
-        # Python versions (>= 3.4) it supports TLS >= 1.0 with SSLv2 and SSLv3
-        # being disabled. For Python > 3.5, PROTOCOL_SSLv23 is an alias for
-        # PROTOCOL_TLS which should be used once compatibility with Python 3.5
-        # is no longer desired.
-
-        if hasattr(ssl, 'PROTOCOL_TLS'):
-            protocol = ssl.PROTOCOL_TLS
-        else:
-            protocol = ssl.PROTOCOL_SSLv23
-
+        protocol = ssl.PROTOCOL_SSLv23
         self.tls_context = ssl.SSLContext(protocol)
         self.tls_context.verify_mode = ssl.CERT_REQUIRED
 
@@ -300,7 +284,7 @@ class TlsServer(BaseServer):
                 "Couldn't bind socket on {}:{}. {}".format(
                     self.socket[0], str(self.socket[1]), e
                 )
-            )
+            ) from e
 
     def handle_request(self, request, client_address):
         logger.debug("New connection from %s", client_address)
